(* producer/consumer example *)

Random.self_init()

(* Mutex pour l'utilisation concurrente de stdout *)
let m = Mutex.create()

let input_buffer = ref []
let output_buffer = ref ""

let out_sched = Fthread.create_scheduler()
let in_sched = Fthread.create_scheduler()

let new_input : unit Fthread.event_t = Fthread.create_event in_sched
let new_output : unit Fthread.event_t = Fthread.create_event out_sched

let process_value value =
  Fthread.unlink();
  (* process value *)
  Mutex.lock m;
  Printf.printf "processing value : %d\n" value; flush stdout;
  Mutex.unlock m;
  let processed_val = string_of_int value in
    Fthread.link out_sched;
    output_buffer := !output_buffer ^ processed_val;
    Fthread.generate new_output;
    Fthread.unlink();
    Fthread.link in_sched

let process () =
  while true
  do
    match !input_buffer with
      | [] -> Fthread.await new_input;
	  if List.length !input_buffer = 0 then Fthread.cooperate()
      | hd::tl -> input_buffer := tl;
	  process_value hd
  done

let produce () =
  for i = 1 to 10
  do
    Fthread.cooperate_n ((Random.int 5)*50);
    Mutex.lock m;
    Printf.printf "produced value : %d\n" i; flush stdout;
    Mutex.unlock m;
    input_buffer := !input_buffer @ [i];
    Fthread.generate new_input;
  done

let consume () =
  while true
  do
    Fthread.await new_output;
    Mutex.lock m;
    Printf.printf "consumed string : %s\n" !output_buffer; flush stdout;
    Mutex.unlock m;
    output_buffer:= "";
    Fthread.cooperate()
  done

let _ =
  (* processor threads creation *)
  for i = 1 to 10
  do
    ignore (Fthread.create_thread in_sched process ())
  done;
  ignore (Fthread.create_thread in_sched produce ());
  ignore (Fthread.create_thread out_sched consume ());
  Fthread.start_scheduler out_sched;
  Fthread.start_scheduler in_sched;
  Fthread.exit()



    
