(** Fair Threads module : mixing cooperative and preemptive threads.*)

(** {6 Types}*)

type 'a event_t 
(** The type of events associated to values of type ['a].*)

and scheduler_t
(** Schedulers type *)
 
type thread_t
(** Threads type *)

(** {6 Exceptions}*)

exception No_value
(** The exception [No_value] is raised by [get_value_n evt k] when
no value is associated to the event [evt] after [k] instants. *)

exception FT_error of string
(** The exception [FT_error] is raised when an error occurs in the Fair Threads
module. *)

(** {6 Constructors} *)

val create_scheduler : unit -> scheduler_t
val create_thread : scheduler_t -> ('a -> unit) -> 'a -> thread_t
val create_event : scheduler_t  -> 'a event_t

val start_scheduler : scheduler_t -> unit

(** {6 Cooperation}*)

val cooperate : unit -> unit
(** return control to the scheduler until the next instant *)

val cooperate_n : int -> unit
(** [cooperate_n] behave like [for i = 1 to n do cooperate done] *)

(** {6 Events} *)

val generate : 'a event_t -> unit
val generate_value : 'a event_t -> 'a -> unit
val await : 'a event_t -> unit
val await_n : 'a event_t -> int -> unit
val get_value : 'a event_t -> 'a
val get_value_n : 'a event_t -> int -> 'a
val link : scheduler_t -> unit
val unlink : unit -> unit

(** {6 Thread control} *)

val thread_stop : thread_t -> unit
val thread_resume : thread_t -> unit
val thread_suspend : thread_t -> unit
val join : thread_t -> unit


(** {6 Exit} *)

val exit : unit -> unit
(** The basic use of [exit] is to terminate the pthread which is running the function main, without exiting from the whole process.*)

(*
(** {6 Debug} *)

val print_sch : scheduler_t -> unit
val print_th : thread_t -> unit
val print_event : id -> unit
*)
